/*
//    FOBO remove control program. Version 1.1
//      This program allows FOBO to be actuated remotely (via USB cable) by a windows desktop application.
//
//    Copyright (C) 2012  Jonathan Dowdall, Project Biped (www.projectbiped.com)
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// this is the library for the servo shield board ... it can be found at http://www.renbotics.com/files/servoshield.zip
// for you program to compile you need to unzip the ServoShield folder and put the whole folder under you "libraries"
// folder inside of your arudino development environment folder ... example "C:\Program Files\arduino-0022\libraries\ServoShield"
#include <ServoShield.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Variables
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ServoShield servos;                              // servoShield object to interact with the servo board

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Constants
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const int  numberOfServos             = 8;      // the communication with the control program is expecting 8 servos 
const int  messageLength              = 19;     // the number of bytes in a message from the control program  
const int  maximumServoShieldPosition = 2000;   // the maximum pulse duration for the servo shield (2ms pulse)
const int  minimumServoShieldPosition = 1000;   // the minimum pulse duration for the servo shield (1ms pulse)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup()
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  //initialize the servos
  InitializeServos();  
  
  //establist a connection with the remote controller (the FOBO poser application)
  EstablistConnection();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void EstablistConnection()
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  //wait for a second to begin (keeps the communication line open in case a new program is being downloaded)
  delay(1000);  
  
  //start up the communication
  Serial.begin(9600);

  //buffer to hold the incoming message
  int inputBuffer[20];
  
  //broadcast our id until someone responds
  while(true)
  {
    //broadcast id
    Serial.print("FOBO");  
    
    //wait for a bit
    delay(100);  
    
    //look for a response
    if(Serial.available() > 1)
    {
      for(int b = 0; b < 2; b++)
        inputBuffer[b] = Serial.read();
        
      // make sure someone friendly is on the line
      if(inputBuffer[0] == (int)'h' && inputBuffer[1] == (int)'i')
      {
            Serial.print("connected");  
            break;
      }
    }      
  }
  
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void InitializeServos()
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
{  
 
  //Initialize the servos (FOBO only uses 8) but the control board could do up to 16
  //Set the minimum and maximum pulse duration  
  for (int servo = 0; servo < numberOfServos; servo++)
    servos.setbounds(servo, minimumServoShieldPosition, maximumServoShieldPosition); 
    
  //Start the servo shield
  servos.start(); 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop()
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
{  
  // check to see if there are enough bytes on the serial line for a message
  if (Serial.available() >= messageLength) 
    // read the incoming message
    if( ReadMessage() )    
      // respond to the computer that is controlling FOBO
      SendResponse();      
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadMessage()
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  // buffer to hold the incoming message
  int inputBuffer[20];

  // read the message
  int index = 0;
  while(Serial.available() > 0)
  {
    inputBuffer[index] = Serial.read();
    index++;
  }

  // make sure the message starts with "cmd" ... otherwise it isn't a valid message
  if(inputBuffer[0] == (int)'c' && inputBuffer[1] == (int)'m' && inputBuffer[2] == (int)'d')
  {
    //set the servo positions
    for (int servo = 0; servo < numberOfServos; servo++)
    {      
      // each servo position is sent as a 2 byte value (high byte, low byte) unsigned integer (from 0 to 65536)
      // this number is encoding the angle of the servo. The number is 100 * the servo angle.  This allows for the
      // storage of 2 significant digits(i.e. the value can be from 0.00 to 120.00 and every value in between).
      // Also remember that the servos FOBO uses have a range of 120 degrees, so the home position is 60 degrees.
      word value = word(inputBuffer[servo*2 + 1 + 3], inputBuffer[servo*2 + 0 + 3]);
      float servoAngle = (float)value/100.00;
      
      // the servo control shield commands the servos via pulse width modulations, not angles
      // the PWMs range from 1000 (equal to 0 degrees) to 2000 (equal to 120 degrees)
      // so the servo angle needs to be converted to the corresponding PWM range.
      int pwm = (int)(servoAngle/120.0* (float)(maximumServoShieldPosition - minimumServoShieldPosition)) + minimumServoShieldPosition;
      
      //Set PWM for the servo shield to send out to the servo
      servos.setposition(servo, pwm);       
    }

    // a valid messgae was received    
    return true;
  }  
  else
    // the message wasn't valid
    return false;  
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SendResponse()
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  // the program is expecting feedback for each the servo position
  // FOBO doesn't have any position feedback, so just send the center position (128)    
  for(int s = 0; s < numberOfServos; s++)
    Serial.print(128, BYTE);
  
  // end the response  
  Serial.print("done");  
}

